
use Exporter;

package commabreak;

@ISA=qw(Exporter);
@EXPORT=qw(commabreak);


# Modified slightly for perl5 package use

#!/usr/local/bin/perl

# Package: CommaBreak
# (commabreak.pl library)

# Andrew Mauer
# 4 March 1994
# 11 July 1994: Example added.
# 13 May 1995: Performance issues section added.
# 4 Jan 1996: Put backslash in front of at-signs (\@) in << OUTPUT
#             to make perl5 happy.

# This routine is used to extract the arguments from a function call. 
# It may be useful in similar contexts, as well.
#
# Usage: &commabreak(STRING,DEPTH);
#   STRING is the string we will be working with.
#   DEPTH is the number of nested parentheses we enter before we start
#   returning arguments. If unspecified, it defaults to 1. A depth
#   of 0 is perfectly acceptable.
#
# We mark the end of a "frame" of parentheses at the requested DEPTH 
# by inserting the string "\000" on the results stack.
#
# To run examples, just comment out the two lines below 
# with the string EXAMPLES in them, and run the library. 
# (That is, do `perl commabreak.pl'.)
#
#
# PERFORMANCE ISSUES: 
# This routine appears to be extremely slow, for reasons that I cannot
# figure out. Here is some profiling information generated using the
# activate.pl script from ADIC on the CSCMDO code, 13 May 95 on goofy.
# (All run with perl5.000.)
#
# Just return from commabreak, before anything.
#        12.8 real        11.3 user         0.6 sys  
# 
# Run through an empty loop over the length of the string, then return.
#        22.3 real        19.0 user         0.7 sys  
# 
# Run through a loop that just does substr($string,$i,1), and then return.
#        32.1 real        28.7 user         0.8 sys  
# 
# Do all logic, but do not put the things you find in @result -- avoids
# some substr's and probably string copies.
#        46.5 real        44.7 user         0.8 sys  
# 
# Loop cost:                7 sec
# Substring cost:           9 sec
# Logic cost:              16 sec
# 
# Total observed slowdown: 26 sec
#

<<EXAMPLES;

$,="\n";

$sample_string = "POINTER (ptr, ptee) , (ptr2, ptee2)";
print "Working with: $sample_string\n";
print "Depth 0:\n";
print &commabreak($sample_string,0);
print "\n";
print "Depth 1:\n";
print &commabreak($sample_string,1);
print "\n";

<<OUTPUT;
Working with: POINTER (ptr, ptee) , (ptr2, ptee2)
Depth 0:
POINTER (ptr, ptee) 
 (ptr2, ptee2)
Depth 1:
ptr
 ptee
     [This is where the \000 was output.]
ptr2
 ptee2
     [This is where the \000 was output.]
OUTPUT

$sample_string = "g_foo(x,g_x(10,12),y)";
print "Working with: $sample_string\n";
print &commabreak($sample_string,1);
print "\n";

<<OUTPUT;
x
g_x(10,12)
y
     [This is where the \000 was output.]
OUTPUT

# This illustrates the defaulting to level 1.
print "Working with: $sample_string\nNot specifying a depth.\n";
print &commabreak($sample_string);
print "\n";

$sample_string = "count_args(x,(y+z),w)";
print "Counting args in: $sample_string\n";
### Uncomment these to use !!! --- done only to make perl5 -w happy
### ALSO REMOVE BACKSLASHES IN FRONT OF "@" sign.
#\@args = &commabreak($sample_string);
#pop(\@args); # Remove \000
print "Found $#args arguments.\n";

<<OUTPUT;
Output not provided since the example was made up late.
OUTPUT

EXAMPLES

# 

sub commabreak {
    local ($string, $target_depth) = @_;

#    package CommaBreak;

    # Make sure nobody messes us up with weird array indexing.
    local ($[) = 0;

    # Keep all the variables super-local.
    local ($depth, $looking_for, $char, $last_comma, $len);

    # Current depth. (Pairs of nested parentheses that we are inside.)
    $depth = 0;

    # This is the separator character that we look for.
    $looking_for = ',';
    
    # This is what we will return.
    local (@result) = ();

    # Don't recompute every loop iteration.
    $len = length($string);

    # Pretend we matched a comma just before the start of the string.
    $last_comma = $[ - 1;

    # If we did not get a depth, assume one.
    if ( ! defined ($target_depth) )
    {
	$target_depth = 1;
    }

    # Start looking through the string.
    for ( $i = 0; $i < $len; $i++)
    {
	$char = substr ($string,$i,1);

	if ( $char eq '(' )
	{
	    $depth++;

	    # Start the first argument with the character after the 
	    # paren which moves us into that depth.

	    if ($depth == $target_depth)
	    {
		$last_comma = $i;
	    }
	}
	elsif ( $char eq ')' )
	{
	    # Deal with the second close paren in foo(bar(x)).
	    if ( $depth == $target_depth )
	    {
		push (@result, substr ($string, 
				       $last_comma + 1,
				       $i - ($last_comma + 1)));

		# This is the line to change to add an indicator of when 
		# a scope is being exited.
		push (@result,"\000");
	    }

	    $depth --;
	}
	elsif (( $char eq $looking_for )
	       && ( $depth == $target_depth ) )
	{
	    push (@result, substr ($string, 
				   $last_comma + 1,
				   $i - ($last_comma + 1)));
	    $last_comma = $i;
	}
    }

    # If we are running off the end of the string, return the last arg.
    
    if ( $depth == $target_depth )
    {
	# From last comma to end of string.
	push (@result, substr ($string, 
			       $last_comma + 1));
    }

    return (@result);
}

1;

